/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *	
 */

// ************************************************************
var kUnknownReasonUrl		= 'about:blank';
var kMAMIUrl			= 'http://xslt.alexa.com/data?cli=17&dat=nsa';

var kNoHTTPReasonUrl		= kMAMIUrl + 'req_type=secure_intranet';
var kSkipDomainReasonUrl	= kMAMIUrl + 'req_type=blocked_list';
var kDataPrefixUrl		= kMAMIUrl;
// ************************************************************


// ************************************************************
// Some globals

var oAsaObserver = null;

// Bufferize the keywords for refresh...
var url_global ="";
// ************************************************************

// ************************************************************
// The observer
function AsaObserver(oPanel) {

  this.m_oPanel = oPanel;
  dump("Init AsaObserver\n");
  
}

// ************************************************************
// The event listener
// ************************************************************
AsaObserver.prototype.observe = function (oSubject, sMessage, sContextUrl)
{

  try {

	dump("Observe " + sContextUrl + "\n");

      	if (oSubject != window._content)
      		// We are only interested in the currently active tab
			return;

      	url_global = sContextUrl;

		// Query Alexa's "What's Related"
		this.alexaRelated(url_global);

  } catch(ex) {}

}

// ************************************************************
AsaObserver.prototype.alexaRelated = function (sContextUrl) {

	dump("Query Alexa for url: " + sContextUrl + "\n");
	var sDataUrl = this.TranslateContext(sContextUrl, true);
	document.getElementById("alexaRelated").setAttribute('src', sDataUrl);

}

AsaObserver.prototype.TranslateContext = function (sUrl, bReferrer) {

	if (!sUrl || ('string' != typeof(sUrl))
        || ('' == sUrl) || sUrl == 'about:blank') {
		return kUnknownReasonUrl;
	}

    // Strip off any query strings (Don't want to be too nosy).
	var nQueryPart = sUrl.indexOf('?');
    if (nQueryPart != 1) {
        sUrl = sUrl.slice(0, nQueryPart);
    }

	// We can only get related links data on HTTP URLs
	if (0 != sUrl.indexOf("http://")) {
		return kNoHTTPReasonUrl;
	}

	// ...and non-intranet sites(those that have a '.' in the domain)
	var sUrlSuffix = sUrl.substr(7);			// strip off "http://" prefix

	var nFirstSlash = sUrlSuffix.indexOf('/');
	var nFirstDot = sUrlSuffix.indexOf('.');

	if (-1 == nFirstDot)
		return kNoHTTPReasonUrl;

	if ((nFirstSlash < nFirstDot) && (-1 != nFirstSlash))
		return kNoHTTPReasonUrl;

	// url is http, non-intranet url: see if the domain is in their blocked list.

	var nPortOffset = sUrlSuffix.indexOf(":");
	var nDomainEnd = (((nPortOffset >=0) && (nPortOffset <= nFirstSlash))
                      ? nPortOffset : nFirstSlash);

	var sDomain = sUrlSuffix;
	if (-1 != nDomainEnd) {
		sDomain = sUrlSuffix.substr(0,nDomainEnd);
	}

	if (DomainInSkipList(sDomain)) {
		return kSkipDomainReasonUrl;
	} else {
		// ok! it is a good url!
		var sFinalUrl = kDataPrefixUrl;
		sFinalUrl += 'cdt='+this.GetCDT(bReferrer);
		sFinalUrl += '&url='+sUrl;
		return sFinalUrl;
	}
}

AsaObserver.prototype.GetCDT = function (bReferrer) {

	var sCDT = '';
	sCDT += 't=' +(bReferrer?'1':'0');
	sCDT += '&pane=nswr6';
	sCDT += '&wid='+this.m_sWindowID;

	return escape(sCDT);
}

function DomainInSkipList(sDomain)
{
	var bSkipDomainFlag = false;

	if ('/' == sDomain[sDomain.length - 1]) {
		sDomain = sDomain.substring(0, sDomain.length - 1);
	}

	try {
		var pref = Components.classes["@mozilla.org/preferences-service;1"]
							 .getService(Components.interfaces.nsIPrefBranch);

		var sDomainList = pref.getCharPref("browser.related.disabledForDomains");
		if ((sDomainList) && (sDomainList != "")) {

			var aDomains = sDomainList.split(",");

			// split on commas
			for (var x=0; x < aDomains.length; x++) {
				var sDomainCopy = sDomain;

				var sTestDomain = aDomains[x];

				if ('*' == sTestDomain[0]) { // wildcard match

                    // strip off the asterisk
					sTestDomain = sTestDomain.substring(1);	
					if (sDomainCopy.length > sTestDomain.length) {
                        var sDomainIndex = sDomain.length - sTestDomain.length;
						sDomainCopy = sDomainCopy.substring(sDomainIndex);
					}
				}

				if (0 == sDomainCopy.lastIndexOf(sTestDomain)) {

					bSkipDomainFlag = true;
					break;
				}
			}
		}
	} catch(ex) {
	}
	return(bSkipDomainFlag);
}
// ************************************************************

function refresh()
{
	if (oAsaObserver) {
		url_global = window._content.document.URL;
	    oAsaObserver.alexaRelated(window._content.document.URL);
	}
}

function timedRefresh()
{
	refresh();
	setTimeout("timedRefresh()", 1000 * 60);
}

function initAsa()
{

  var oObsService = Components.classes["@mozilla.org/observer-service;1"].getService();
  oObsInterface = oObsService.QueryInterface(Components.interfaces.nsIObserverService);

  oAsaObserver = new AsaObserver(document);

  if (oObsInterface && oAsaObserver) {
    oObsInterface.addObserver(oAsaObserver, "EndDocumentLoad", false);
  }

  

  //  timedRefresh();
  refresh();

}

function killAsa()
{
	var oObsService = Components.classes["@mozilla.org/observer-service;1"].getService();
	oObsInterface = oObsService.QueryInterface(Components.interfaces.nsIObserverService);
	if (oObsInterface && oAsaObserver) {
		oObsInterface.removeObserver(oAsaObserver, "EndDocumentLoad");
		oAsaObserver = null;
	}
}

addEventListener("load", initAsa, false);
addEventListener("unload", killAsa, false);
